@@ -5,15 +5,26 @@ import code
 import six
 import sys
 import threading
+import time
 import websocket
+from six.moves.urllib.parse import urlparse
 try:
     import readline
 except:
     pass
 
 
+def get_encoding():
+    encoding = getattr(sys.stdin, "encoding", "")
+    if not encoding:
+        return "utf-8"
+    else:
+        return encoding.lower()
+
+
 OPCODE_DATA = (websocket.ABNF.OPCODE_TEXT, websocket.ABNF.OPCODE_BINARY)
-ENCODING = getattr(sys.stdin, "encoding", "").lower()
+ENCODING = get_encoding()
+
 
 class VAction(argparse.Action):
     def __call__(self, parser, args, values, option_string=None):
@@ -29,24 +40,30 @@ def parse_args():
     parser = argparse.ArgumentParser(description="WebSocket Simple Dump Tool")
     parser.add_argument("url", metavar="ws_url",
                         help="websocket url. ex. ws://echo.websocket.org/")
+    parser.add_argument("-p", "--proxy",
+                        help="proxy url. ex. http://127.0.0.1:8080")
     parser.add_argument("-v", "--verbose", default=0, nargs='?', action=VAction,
                         dest="verbose",
                         help="set verbose mode. If set to 1, show opcode. "
                         "If set to 2, enable to trace  websocket module")
     parser.add_argument("-n", "--nocert", action='store_true',
                         help="Ignore invalid SSL cert")
+    parser.add_argument("-r", "--raw", action="store_true",
+                        help="raw output")
+    parser.add_argument("-s", "--subprotocols", nargs='*',
+                        help="Set subprotocols")
+    parser.add_argument("-o", "--origin",
+                        help="Set origin")
+    parser.add_argument("--eof-wait", default=0, type=int,
+                        help="wait time(second) after 'EOF' received.")
+    parser.add_argument("-t", "--text",
+                        help="Send initial text")
+    parser.add_argument("--timings", action="store_true",
+                        help="Print timings in seconds")
 
     return parser.parse_args()
 
-
-class InteractiveConsole(code.InteractiveConsole):
-    def write(self, data):
-        sys.stdout.write("\033[2K\033[E")
-        # sys.stdout.write("\n")
-        sys.stdout.write("\033[34m" + data + "\033[39m")
-        sys.stdout.write("\n> ")
-        sys.stdout.flush()
-
+class RawInput():
     def raw_input(self, prompt):
         if six.PY3:
             line = input(prompt)
@@ -60,20 +77,55 @@ class InteractiveConsole(code.InteractiveConsole):
 
         return line
 
+class InteractiveConsole(RawInput, code.InteractiveConsole):
+    def write(self, data):
+        sys.stdout.write("\033[2K\033[E")
+        # sys.stdout.write("\n")
+        sys.stdout.write("\033[34m< " + data + "\033[39m")
+        sys.stdout.write("\n> ")
+        sys.stdout.flush()
+
+    def read(self):
+        return self.raw_input("> ")
+
+class NonInteractive(RawInput):
+    def write(self, data):
+        sys.stdout.write(data)
+        sys.stdout.write("\n")
+        sys.stdout.flush()
+
+    def read(self):
+        return self.raw_input("")
 
 def main():
+    start_time = time.time()
     args = parse_args()
-    console = InteractiveConsole()
     if args.verbose > 1:
         websocket.enableTrace(True)
+    options = {}
+    if (args.proxy):
+        p = urlparse(args.proxy)
+        options["http_proxy_host"] = p.hostname
+        options["http_proxy_port"] = p.port
+    if (args.origin):
+        options["origin"] = args.origin
+    if (args.subprotocols):
+        options["subprotocols"] = args.subprotocols
     opts = {}
     if (args.nocert):
         opts = { "cert_reqs": websocket.ssl.CERT_NONE, "check_hostname": False }
-    ws = websocket.create_connection(args.url, sslopt=opts)
-    print("Press Ctrl+C to quit")
+    ws = websocket.create_connection(args.url, sslopt=opts, **options)
+    if args.raw:
+        console = NonInteractive()
+    else:
+        console = InteractiveConsole()
+        print("Press Ctrl+C to quit")
 
     def recv():
-        frame = ws.recv_frame()
+        try:
+            frame = ws.recv_frame()
+        except websocket.WebSocketException:
+            return (websocket.ABNF.OPCODE_CLOSE, None)
         if not frame:
             raise websocket.WebSocketException("Not a valid frame %s" % frame)
         elif frame.opcode in OPCODE_DATA:
@@ -82,7 +134,7 @@ def main():
             ws.send_close()
             return (frame.opcode, None)
         elif frame.opcode == websocket.ABNF.OPCODE_PING:
-            ws.pong("Hi!")
+            ws.pong(frame.data)
             return frame.opcode, frame.data
 
         return frame.opcode, frame.data
@@ -92,25 +144,37 @@ def main():
         while True:
             opcode, data = recv()
             msg = None
+            if six.PY3 and opcode == websocket.ABNF.OPCODE_TEXT and isinstance(data, bytes):
+                data = str(data, "utf-8")
             if not args.verbose and opcode in OPCODE_DATA:
-                msg = "< %s" % data
+                msg = data
             elif args.verbose:
-                msg = "< %s: %s" % (websocket.ABNF.OPCODE_MAP.get(opcode), data)
+                msg = "%s: %s" % (websocket.ABNF.OPCODE_MAP.get(opcode), data)
 
-            if msg:
-                console.write(msg)
+            if msg is not None:
+                if (args.timings):
+                    console.write(str(time.time() - start_time) + ": " + msg)
+                else:
+                    console.write(msg)
+
+            if opcode == websocket.ABNF.OPCODE_CLOSE:
+                break
 
     thread = threading.Thread(target=recv_ws)
     thread.daemon = True
     thread.start()
 
+    if args.text:
+        ws.send(args.text)
+
     while True:
         try:
-            message = console.raw_input("> ")
+            message = console.read()
             ws.send(message)
         except KeyboardInterrupt:
             return
         except EOFError:
+            time.sleep(args.eof_wait)
             return
 
 
